import unittest
from pcalib.py.algorithms import Matrix, pca, explained_variance_ratio, auto_select_k, reconstruction_error, \
    reconstruct_from_pca, plot_pca_projection


class TestPCA(unittest.TestCase):
    def setUp(self):
        self.X = Matrix([[1, 2], [3, 4], [5, 6]])

    def test_pca_projection(self):
        X_proj, gamma, *_ = pca(self.X, k=1)
        mean_proj = sum(row[0] for row in X_proj.data) / len(X_proj.data)

        self.assertAlmostEqual(mean_proj, 0.0, places=5)
        self.assertTrue(0 <= gamma <= 1)

    def test_explained_variance_ratio(self):
        vals = [3, 1]

        self.assertAlmostEqual(explained_variance_ratio(vals, 1), 0.75, places=5)

    def test_auto_select_k(self):
        vals = [3, 1, 0.5]
        k = auto_select_k(vals, threshold=0.8)

        self.assertEqual(k, 2)

    def test_reconstruction_error(self):
        X_proj, *_ = pca(self.X, k=1)
        X_recon = reconstruct_from_pca(X_proj, self.X, k=1)
        err = reconstruction_error(self.X, X_recon)

        self.assertTrue(err > 0)
        self.assertTrue(err < 5)

    def test_plot_pca_projection(self):
        X_proj, *_ = pca(self.X, k=2)
        fig = plot_pca_projection(X_proj)

        self.assertTrue(hasattr(fig, 'savefig'))


if __name__ == '__main__':
    unittest.main()



import unittest
from pcalib.py.algorithms import Matrix, gauss_solver, center_data, covariance_matrix, find_eigenvalues, \
    find_eigenvectors


class TestLinalg(unittest.TestCase):
    def test_gauss_solver(self):
        A = Matrix([[2, 1], [1, 3]])
        b = [5, 10]
        x = gauss_solver(A, b)

        self.assertAlmostEqual(x[0], 1.0, places=5)
        self.assertAlmostEqual(x[1], 3.0, places=5)

    def test_center_data(self):
        X = Matrix([[1, 2], [3, 4]])
        Xc = center_data(X)

        self.assertAlmostEqual(Xc.data[0][0], -1.0)
        self.assertAlmostEqual(Xc.data[0][1], -1.0)
        self.assertAlmostEqual(Xc.data[1][0], 1.0)
        self.assertAlmostEqual(Xc.data[1][1], 1.0)

    def test_covariance_matrix(self):
        X = Matrix([[1, 2], [3, 4]])
        Xc = center_data(X)
        C = covariance_matrix(Xc)

        self.assertAlmostEqual(C.data[0][0], 2.0)
        self.assertAlmostEqual(C.data[0][1], 2.0)
        self.assertAlmostEqual(C.data[1][0], 2.0)
        self.assertAlmostEqual(C.data[1][1], 2.0)

    def test_find_eigenvalues_and_vectors(self):
        C = Matrix([[2, 0], [0, 1]])
        eigenvalues = find_eigenvalues(C)
        eigenvalues_sorted = sorted(eigenvalues)

        self.assertAlmostEqual(eigenvalues_sorted[0], 1.0, places=5)
        self.assertAlmostEqual(eigenvalues_sorted[1], 2.0, places=5)

        eigenvectors = find_eigenvectors(C, eigenvalues_sorted)
        for i, val in enumerate(eigenvalues_sorted):
            v = [eigenvectors[i].data[0][0], eigenvectors[i].data[1][0]]
            Av = [C.data[0][0] * v[0] + C.data[0][1] * v[1], C.data[1][0] * v[0] + C.data[1][1] * v[1]]
            lv = [val * v[0], val * v[1]]

            self.assertAlmostEqual(Av[0], lv[0], places=5)
            self.assertAlmostEqual(Av[1], lv[1], places=5)


if __name__ == '__main__':
    unittest.main()



from setuptools import setup, find_packages

setup(
    name="pcalib",
    version="0.1.1",
    description="PCA and linear algebra library with C++ backend",
    author="Selim Poladov",
    author_email="main@netherite.ru",
    packages=find_packages(),
    install_requires=[
        "matplotlib",
        "scikit-learn"
    ],
    include_package_data=True,
    package_data={
        "pcalib": ["py/*.py", "cpp/*.dylib"],
    },
    python_requires=">=3.8",
)